static void quit_button (GtkWidget * widget, gpointer data)
{
    gtk_main_quit ();
}


static void widget_changed (GtkWidget *widget, CompizSettingsOption * option)
{

  gchar * widget_type = (gchar *) gtk_widget_get_name (widget);

   if (option->type == CompizSettingsOptionAction) {

	CompizSettingsActionKey * action = (CompizSettingsActionKey *)option->data;

	if (!strcmp(widget_type, "ActionKey")) {
	  action->key = (gchar *) gtk_entry_get_text (GTK_ENTRY(widget));
	} else if (!strcmp(widget_type, "ActionButton")) {
	  action->button = (gchar *) gtk_entry_get_text (GTK_ENTRY(widget));
	} else { /* Edge List */

	  gchar * list_value = str_replace(widget_type, "EdgeCheckbox", "");

	   //action->edge = (GSList *) backend_get_option(option->plugin_name, option->screen, option->key, option->type);

    	   gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
    	   if (!value) {
	     action->edge = remove_from_gslist(action->edge, list_value);
    	   } else {
	     action->edge = add_to_gslist(action->edge, list_value);
	   }
        }

     backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)action, (int *) option->type);

   } else if (!strcmp(widget_type, "GtkEntry")) {
     gchar * string = (gchar *) gtk_entry_get_text (GTK_ENTRY(widget));
     backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)string, (int *) option->type);
   } else if (!strcmp(widget_type, "GtkCheckButton") | !strcmp(widget_type, "GtkRadioButton")) {
    gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
    backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)value, (int *) option->type);
   } else if (!strcmp(widget_type, "GtkHScale")) {
	//printf("value: %f\n", gtk_range_get_value (GTK_RANGE(widget)));
	//Hscale is special because we don't know if it is a float or int value
	if (gtk_scale_get_digits(GTK_SCALE(widget)) == 0) {
    	 int value = gtk_range_get_value (GTK_RANGE(widget));
	 backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)value, (int *) option->type);
	} else {
	 //gcc won't convert float to pointer
    	 gdouble value = gtk_range_get_value (GTK_RANGE(widget));
	 const char * float_value = g_strdup_printf("%f", value);
	 //printf("value: %s\n", float_value);
	 backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)float_value, (int *) option->type);
	}
    //backend_set_option(data, (gpointer *)value, GINT_TO_POINTER(CompizSettingsOptionBool));
   } else if (strmatch(widget_type, "EdgeCheckbox")) {
     
	GSList * list;
	gchar * list_value = str_replace(widget_type, "EdgeCheckbox", "");
	list = (GSList *) backend_get_option(option->plugin_name, option->screen, option->key, option->type);

    	gboolean value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
    	if (!value) {
	  list = remove_from_gslist(list, list_value);
    	} else {
	  list = add_to_gslist(list, list_value);
	}

    	backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)list, (int *) option->type);

	g_slist_free(list);

   } else if (!strcmp(widget_type, "GtkComboBox")) {
	gchar * string = gtk_combo_box_get_active_text (GTK_COMBO_BOX(widget));
     backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)string, (int *) option->type);
	g_free(string);
   } else if (!strcmp(widget_type, "GtkColorButton")) {
	GdkColor *color = malloc(sizeof(GdkColor));
	gtk_color_button_get_color (GTK_COLOR_BUTTON(widget), color);
	gchar * string = gdkcolor_to_hex(color);
     backend_set_option(option->plugin_name, option->screen, option->key, (gpointer *)string, (int *) option->type);
	g_free(string);
   }  else {
    printf("Widget %s not supported, this should never happen\n", widget_type);
  }
}

void active_plugins_changed_checkbox(GConfClient *client,
		    guint        signal,
		    GConfEntry  *entry,
		    Plugin_Filter_Button * user_data){ 


    g_slist_free(active_plugins);
    active_plugins = backend_get_list_of_active_plugins();
	if (data_is_in_gslist(active_plugins, user_data->plugin_name)) {
	   /* Plugin is active */
	   if (user_data->checkbox_active == FALSE) { /* Only bother updating if status has changed */
	      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data->checkbox), TRUE);
	      user_data->checkbox_active = TRUE;
	   }
	} else {
	   if (user_data->checkbox_active == TRUE) { /* Only bother updating if status has changed */
	      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(user_data->checkbox), FALSE);
	      user_data->checkbox_active = FALSE;
	   }
	}

}

GtkWidget * new_slider(float min, float max, float decimals, float increments, CompizSettingsOption * option) {
	GtkObject *adj;
  	GtkWidget * widget;
	float start;
 

	if (option->type == CompizSettingsOptionFloat) {
	  start = (float) ((int)option->data / 1000);
	} else if (option->type == CompizSettingsOptionInt) {
          start = (int) option->data;
	}
	
	adj = gtk_adjustment_new(start,min,max,increments,increments,0);
	widget = gtk_hscale_new(GTK_ADJUSTMENT(adj));
        gtk_scale_set_value_pos(GTK_SCALE(widget),GTK_POS_RIGHT);
	gtk_scale_set_digits(GTK_SCALE(widget),decimals);
        gtk_widget_set_size_request(widget,200,-1);

	   gtk_range_set_update_policy(GTK_RANGE (widget), GTK_UPDATE_DISCONTINUOUS);


	//backend_create_new_callback(key, (GFunc *) backend_setting_changed_externally, widget);
	g_signal_connect (GTK_RANGE (widget), "value_changed", G_CALLBACK(widget_changed), option);


	return widget;
  }

GtkWidget * new_section_title(gchar * text) {
 GtkWidget * widget;

  widget = gtk_label_new (g_strdup_printf("<span size=\"x-large\">%s</span>", text));
  gtk_misc_set_padding (GTK_MISC (widget), 0, 5);
    GdkColor color;
    gdk_color_parse ("#b55623", &color);

  GtkStyle * widget_style = gtk_style_copy(widget->style);  
  widget_style->font_desc = pango_font_description_from_string("Nimbus Sans L 9");
  widget_style->fg[GTK_STATE_NORMAL] = (GdkColor) color;
  gtk_widget_set_style(widget,widget_style);

  gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
  gtk_misc_set_alignment (GTK_MISC (widget), 0, 0);

  return widget;

}

GtkWidget * gtk_icon_button_new(gchar * label_text, gchar * stock_image, GFunc * action) {
	GtkWidget * button;
	GtkWidget * label;
	GtkWidget * hbox;

	 button = gtk_button_new();

	 hbox = gtk_hbox_new (FALSE, 0);
  	 gtk_container_add (GTK_CONTAINER (button), hbox);

	if (stock_image != NULL) {
	   GtkWidget * image;
	   image = gtk_image_new_from_stock (stock_image, GTK_ICON_SIZE_BUTTON);
  	 gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
	}

	label = gtk_label_new (label_text);
  	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

  	 g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (action), label);

  return button;
}


GtkWidget * new_label_button(int indent, gchar * label_text, GFunc * action, gchar * stock_image) {
 GtkWidget * hbox;
 GtkWidget * widget;
 GtkWidget * label;

 hbox = gtk_hbox_new (FALSE, 0);


  	  widget = gtk_icon_button_new(label_text, stock_image, action);


  	 gtk_button_set_relief (GTK_BUTTON (widget), GTK_RELIEF_NONE);

  GtkWidget * alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_set_size_request (alignment, indent, -1);

  gtk_box_pack_start (GTK_BOX (hbox), alignment, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);

  return hbox;
}


GtkWidget * color_button_new(CompizSettingsOption * option) {

    GdkColor * color = malloc(sizeof(GdkColor));

 
	GtkWidget * widget = NULL;
	/* Basic check it's a valid hex */
	if (option->data != NULL && strlen((gchar *) option->data) == 7) {
          gdk_color_parse ((gchar *) option->data, color);
	  widget = gtk_color_button_new_with_color(color);
	} else {
	  widget = gtk_color_button_new();

	}

   return widget;

}

GtkWidget * new_checkbox_with_label(CompizSettingsOption * option, gchar *label, gint * edge_type) {
	GtkWidget * widget;
	gboolean checked = FALSE;
	gchar *match_key = NULL;

        widget = gtk_check_button_new_with_mnemonic(label);

	/* Support for edge buttons as gconf is a list */
	if (option->type == CompizSettingsOptionAction) {
	 /* I don't like this AT ALL */

	  //edges = (GSList *) backend_get_option(key, GINT_TO_POINTER(CompizSettingsOptionStringList));


		if (edge_type ==GINT_TO_POINTER(EdgeLeft)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxLeft");
		 match_key = g_strdup_printf("Left");
		} else if (edge_type ==GINT_TO_POINTER(EdgeRight)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxRight");
		 match_key = g_strdup_printf("Right");
		} else if (edge_type ==GINT_TO_POINTER(EdgeTop)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxTop");
		 match_key = g_strdup_printf("Top");
		} else if (edge_type ==GINT_TO_POINTER(EdgeBottom)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxBottom");
		 match_key = g_strdup_printf("Bottom");
		} else if (edge_type ==GINT_TO_POINTER(EdgeBottomRight)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxBottomRight");
		 match_key = g_strdup_printf("BottomRight");
		} else if (edge_type ==GINT_TO_POINTER(EdgeTopRight)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxTopRight");
		 match_key = g_strdup_printf("TopRight");
		} else if (edge_type ==GINT_TO_POINTER(EdgeBottomLeft)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxBottomLeft");
		 match_key = g_strdup_printf("BottomLeft");
		} else if (edge_type ==GINT_TO_POINTER(EdgeTopRight)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxTopRight");
		 match_key = g_strdup_printf("TopRight");
		} else if (edge_type ==GINT_TO_POINTER(EdgeTopLeft)) {
	 	 gtk_widget_set_name (GTK_WIDGET (widget), "EdgeCheckboxTopLeft");
		 match_key = g_strdup_printf("TopLeft");
		}

	   CompizSettingsActionKey * action = (CompizSettingsActionKey *) option->data;
	   if (data_is_in_gslist(action->edge, match_key)) {
	     checked = TRUE;
	   }

	
	  g_free(match_key);


	} else {
         // checked = (gboolean) backend_get_option(key, GINT_TO_POINTER(CompizSettingsOptionBool));
	}




	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), checked);
	//  backend_create_new_callback(key, (GFunc *) backend_setting_changed_externally, widget);
	  g_signal_connect (GTK_WIDGET (widget), "toggled", G_CALLBACK(widget_changed), option);


	return widget;
 }

GtkWidget * new_action_widget(CompizSettingsOption * option) {
	GtkWidget * widget;
	GtkWidget * textentry;
        GtkWidget * main_vbox;
        GtkWidget * vbox;
        GtkWidget * hbox;
        GtkWidget * hbox2;
        GtkWidget * expander;
        GtkWidget * frame;

   frame = gtk_frame_new(NULL);
 	        gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
		gtk_container_set_border_width (GTK_CONTAINER(frame), 2);
  main_vbox = gtk_hbox_new (TRUE, 2);

  gtk_container_add (GTK_CONTAINER (frame), main_vbox);

       gchar * label = key_to_human_readable(option->key);
        expander = gtk_expander_new (label);
    gtk_box_pack_start (GTK_BOX (main_vbox), expander, FALSE, TRUE, 0);

  vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (expander), vbox);

 /* Keyboard Shortcuts */
    	hbox2 = gtk_hbox_new (FALSE, 2);
    	hbox = gtk_hbox_new (FALSE, 2);
          gtk_box_pack_start (GTK_BOX (hbox2), hbox, TRUE, FALSE, 0);

        widget = gtk_label_new (_("Keyboard Shortcut"));
  	  gtk_widget_set_size_request (widget, 150, -1);
  	  gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
          gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, TRUE, 0);

  	textentry = gtk_entry_new ();
  	  gtk_widget_set_size_request (textentry, 300, -1);
  	  gtk_editable_set_editable (GTK_EDITABLE (textentry), FALSE);
	  gtk_widget_set_name (GTK_WIDGET (textentry), "ActionKey");
	  CompizSettingsActionKey * action = (CompizSettingsActionKey *) option->data;
	  gtk_entry_set_text(GTK_ENTRY (textentry), (gchar *)action->key);
          gtk_box_pack_start (GTK_BOX (hbox), textentry, FALSE, TRUE, 0);
	g_signal_connect (GTK_WIDGET (textentry), "changed", G_CALLBACK(widget_changed), option);

	  widget = gtk_button_new_with_label ("Grab Keys");
  	  gtk_widget_set_size_request (widget, 150, -1);
          gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, TRUE, 0);
        g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (keyboard_grab_key_pressed),  textentry);

   gtk_box_pack_start (GTK_BOX (vbox), hbox2, TRUE, TRUE, 0);
  /* End Keyboard shortcuts */

 /* Mouse Shortcuts */

    	hbox2 = gtk_hbox_new (FALSE, 2);
    	hbox = gtk_hbox_new (FALSE, 2);
          gtk_box_pack_start (GTK_BOX (hbox2), hbox, TRUE, FALSE, 0);

        widget = gtk_label_new (_("Mouse Shortcut"));
  	  gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
  	  gtk_widget_set_size_request (widget, 150, -1);
          gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, TRUE, 0);

  	textentry = gtk_entry_new ();
  	  gtk_widget_set_size_request (textentry, 300, -1);
  	  gtk_editable_set_editable (GTK_EDITABLE (textentry), FALSE);
	  gtk_widget_set_name (GTK_WIDGET (textentry), "ActionButton");
	  gtk_entry_set_text(GTK_ENTRY (textentry), (gchar *)action->button);
          gtk_box_pack_start (GTK_BOX (hbox), textentry, FALSE, TRUE, 0);
	g_signal_connect (GTK_WIDGET (textentry), "changed", G_CALLBACK(widget_changed), option);

	  widget = gtk_button_new_with_label ("Grab Mouse");
  	  gtk_widget_set_size_request (widget, 150, -1);
          gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, TRUE, 0);
        g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (mouse_grab_key_pressed),  textentry);

   gtk_box_pack_start (GTK_BOX (vbox), hbox2, TRUE, TRUE, 0);
  /* End Mouse shortcuts */

  /* Edges */

  GtkWidget *image1;
  GtkWidget * table1;
  GtkWidget * edgehbox;


   hbox2 = gtk_hbox_new (TRUE, 2);
   gtk_box_pack_start (GTK_BOX (vbox), hbox2, TRUE, TRUE, 0);

   edgehbox = gtk_hbox_new (FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox2), edgehbox, FALSE, FALSE, 0);

   table1 = gtk_table_new (3, 3, FALSE);
  	  gtk_widget_set_size_request (table1, 450, -1);

        widget = gtk_label_new (_("Screen Edge"));
  	gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
  	gtk_widget_set_size_request (widget, 150, -1);
	gtk_box_pack_start (GTK_BOX (edgehbox), widget, FALSE, FALSE, 0);
  
	gtk_box_pack_start (GTK_BOX (edgehbox), table1, FALSE, FALSE, 0);

  image1 = create_pixmap (window1, "edges_desktop_screenshot.jpg");
  gtk_table_attach (GTK_TABLE (table1), image1, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_padding (GTK_MISC (image1), 5, 5);

  widget = new_checkbox_with_label(option, "Top Right", GINT_TO_POINTER(EdgeTopRight));
  gtk_table_attach (GTK_TABLE (table1), widget, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Top Left", GINT_TO_POINTER(EdgeTopLeft));
  gtk_table_attach (GTK_TABLE (table1), widget, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Bottom Right", GINT_TO_POINTER(EdgeBottomRight));
  gtk_table_attach (GTK_TABLE (table1), widget, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Bottom Left", GINT_TO_POINTER(EdgeBottomLeft));
  gtk_table_attach (GTK_TABLE (table1), widget, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Top", GINT_TO_POINTER(EdgeTop));
  gtk_table_attach (GTK_TABLE (table1), widget, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Bottom", GINT_TO_POINTER(EdgeBottom));
  gtk_table_attach (GTK_TABLE (table1), widget, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Right", GINT_TO_POINTER(EdgeRight));
  gtk_table_attach (GTK_TABLE (table1), widget, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  widget = new_checkbox_with_label(option, "Left", GINT_TO_POINTER(EdgeLeft));
  gtk_table_attach (GTK_TABLE (table1), widget, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);



  return frame;
}

GtkWidget * new_textbox(gint * limit, CompizSettingsOption * option) {
	GtkWidget * widget;

  	widget = gtk_entry_new_with_max_length ((gint)limit);

	  gtk_entry_set_text(GTK_ENTRY (widget), (gchar *)option->data);
	

	//printf("Nofification added: %s\n", key);

	//backend_create_new_callback(key, (GFunc *) backend_setting_changed_externally, widget);
	g_signal_connect (GTK_WIDGET (widget), "changed", G_CALLBACK(widget_changed), option);
	

	return widget;
  }




GtkWidget * new_checkbox(CompizSettingsOption * option) {
	GtkWidget * widget;

          widget = gtk_check_button_new();
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), (gboolean) option->data);
	 // backend_create_new_callback(key, (GFunc *) backend_setting_changed_externally, widget);
	  g_signal_connect (GTK_WIDGET (widget), "toggled", G_CALLBACK(widget_changed), option);
	return widget;
 }


GtkWidget * new_radio(gchar *key, GSList * radiobutton_group, GFunc * callback) {
  gboolean checked;
         // checked = (gboolean) backend_get_option(key, GINT_TO_POINTER(CompizSettingsOptionBool));
  GtkWidget * radiobutton = gtk_radio_button_new (NULL);

  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton), radiobutton_group);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radiobutton), checked);
  //backend_create_new_callback(key, (GFunc *) callback, radiobutton);
  //g_signal_connect (GTK_WIDGET (radiobutton), "toggled", G_CALLBACK(widget_changed), key);

  return radiobutton;
}


void gtk_combo_box_append_list(GSList * string_restriction, GtkWidget * widget, gchar * active_string) {
int number = 0;
  while (string_restriction)
    {
      gtk_combo_box_append_text (GTK_COMBO_BOX(widget), string_restriction->data);
	if (!strcmp(string_restriction->data, active_string)) {
	   gtk_combo_box_set_active (GTK_COMBO_BOX(widget), number);
	}
      string_restriction = string_restriction->next;
      number++;
    }

}

GtkWidget * new_combo_box(GSList * string_restriction, CompizSettingsOption * option) {

  GtkWidget * combobox = gtk_combo_box_new_text ();
  gtk_combo_box_append_list(string_restriction, combobox, (gchar *) option->data);
  //backend_create_new_callback(key, (GFunc *) backend_setting_changed_externally, combobox);
  g_signal_connect (GTK_WIDGET (combobox), "changed", G_CALLBACK(widget_changed), option);

  return combobox;
}

void option_to_widget(CompizSettingsOption * option, GtkWidget *vbox) {
          
		  gchar * option_name;
	          gchar * screen = option->screen;
	          gchar * plugin_name = option->plugin_name;
		  gchar * key_name = option->key;
		  gint option_type = option->type;

		  option_name = key_to_human_readable(key_name);


     		 	if (option_type == CompizSettingsOptionString) {
			  GtkWidget * widget;
			  GtkWidget * textbox_vbox = gtk_hbox_new (TRUE, 2);

    				  GSList * string_restriction = NULL;
    				  string_restriction = backend_get_string_restriction(plugin_name, screen, key_name);
			  	
			  	widget = gtk_label_new(option_name);
  				gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
				gtk_misc_set_padding (GTK_MISC (widget), 50, 0);

			  gtk_box_pack_start (GTK_BOX (textbox_vbox), widget, FALSE, TRUE, 0);


				if (string_restriction == NULL) {
			  	   widget = new_textbox(NULL, option);
				} else {
				   widget = new_combo_box(string_restriction, option);
				}
			        gtk_box_pack_start (GTK_BOX (textbox_vbox), widget, TRUE, TRUE, 0);
		      	  gtk_box_pack_start (GTK_BOX (vbox), textbox_vbox, FALSE, TRUE, 0);

    		 	} else if (option_type == CompizSettingsOptionColor) {

			  GtkWidget * widget;
			  GtkWidget * checkbox_vbox = gtk_hbox_new (TRUE, 2);
	
			  	widget = gtk_label_new(option_name);
  			        gtk_widget_set_size_request (widget, -1, 20);
  				gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
				gtk_misc_set_padding (GTK_MISC (widget), 50, 0);
			  	gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

				widget = color_button_new (option);
 	 	        g_signal_connect (GTK_OBJECT (widget), "color_set", G_CALLBACK(widget_changed), option);
			  	gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, FALSE, FALSE, 0);

		      	  gtk_box_pack_start (GTK_BOX (vbox), checkbox_vbox, FALSE, FALSE, 0);
    		 	} else if (option_type == CompizSettingsOptionBool) {
			  GtkWidget * widget;
			  GtkWidget * checkbox_vbox = gtk_hbox_new (TRUE, 2);
	
			  	widget = gtk_label_new(option_name);
  			        gtk_widget_set_size_request (widget, -1, 20);
  				gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
				gtk_misc_set_padding (GTK_MISC (widget), 50, 0);
			  	gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

			  	widget = new_checkbox(option);
			        gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

		      	  gtk_box_pack_start (GTK_BOX (vbox), checkbox_vbox, FALSE, FALSE, 0);
    		 	} else if (option_type == CompizSettingsOptionInt) {
				  GtkWidget * widget;
				  GtkWidget * checkbox_vbox = gtk_hbox_new (TRUE, 2);

    				  CompOptionIntRestriction * int_restriction = NULL;
    				  int_restriction = backend_get_int_restriction(plugin_name, screen, key_name);

				  	widget = gtk_label_new(option_name);
  			        	gtk_widget_set_size_request (widget, -1, 20);
  					gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
					gtk_misc_set_padding (GTK_MISC (widget), 50, 0);
				  	gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

				if (int_restriction != NULL) {
				  	widget = new_slider(int_restriction->min, int_restriction->max, 0, 0, option);
				} else {
				  	widget = new_slider(0, 100, 0, 0, option);
				}

				        gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

			      	  gtk_box_pack_start (GTK_BOX (vbox), checkbox_vbox, FALSE, FALSE, 0);
    		 	} else if (option_type == CompizSettingsOptionFloat) {
				  GtkWidget * widget;
				  GtkWidget * checkbox_vbox = gtk_hbox_new (TRUE, 2);

    				  CompOptionFloatRestriction * float_restriction = NULL;
    				  float_restriction = backend_get_float_restriction(plugin_name, screen, key_name);
		
				  	widget = gtk_label_new(option_name);
  					gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
					gtk_misc_set_padding (GTK_MISC (widget), 50, 0);
				  	gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

				if (float_restriction != NULL) {
				  	widget = new_slider(float_restriction->min, float_restriction->max, 2, float_restriction->precision, option);
				} else {
				  	widget = new_slider(0, 100, 2, 0, option);
				}

				        gtk_box_pack_start (GTK_BOX (checkbox_vbox), widget, TRUE, TRUE, 0);

			      	  gtk_box_pack_start (GTK_BOX (vbox), checkbox_vbox, FALSE, FALSE, 0);
    		 	} else if (option_type == CompizSettingsOptionAction) {
			  GtkWidget * textbox_vbox = gtk_hbox_new (TRUE, 2);

			  	 GtkWidget * action_box = new_action_widget(option);
  					//gtk_entry_set_has_frame (GTK_ENTRY (textentry), FALSE);

			        gtk_box_pack_start (GTK_BOX (textbox_vbox), action_box, TRUE, TRUE, 0);

		      	  gtk_box_pack_start (GTK_BOX (vbox), textbox_vbox, FALSE, TRUE, 0);
    		 	}else {
			 // printf("No widget to handle option type\n");
		          return;
		 	}

		  //g_free(key_name);
		  g_free(option_name);
		}


GtkWidget * build_pane(gchar * plugin_name, GSList * allscreen_options_list,  GSList * screen_options_list, gint * option_type, GSList * must_not_contain, GSList * must_contain, gchar * title) {
    	GtkWidget * frame;
	GtkWidget * vbox;
	GtkWidget * hbox;
	GtkWidget * vbox1;
	GtkWidget * label;

	hbox = gtk_hbox_new (TRUE, 0);
	vbox = gtk_vbox_new (FALSE, 0);
    	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, TRUE, 5);


	label = new_section_title(title);
      	 gtk_widget_show_all (label);
  	 gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);


	//Grab list of plugin options and store in GSList

	if (allscreen_options_list != NULL) {
    	frame = gtk_frame_new("AllScreen Options");
	
    	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
	    vbox1 = gtk_vbox_new (FALSE, 0);
	    gtk_container_add (GTK_CONTAINER(frame), vbox1); 
		for (; allscreen_options_list; allscreen_options_list = allscreen_options_list->next) {
	   	  option_to_widget(allscreen_options_list->data, vbox1);
		}

	}

	if (screen_options_list != NULL) {
    	frame = gtk_frame_new("Screen Options");
    	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
	    vbox1 = gtk_vbox_new (FALSE, 0);
	    gtk_container_add (GTK_CONTAINER(frame), vbox1); 

		for (; screen_options_list; screen_options_list = screen_options_list->next) {
	   	  option_to_widget(screen_options_list->data, vbox1);
		}
	}


   return hbox;
}

GtkWidget * setup_new_notebook_page(GtkWidget * notebook, gchar *label, gint page) {
 GtkWidget *vbox;
 GtkWidget *scroll;
 GtkWidget *viewport;
 GtkWidget *widget;

 vbox = gtk_vbox_new (TRUE, 0);
  gtk_container_add (GTK_CONTAINER (notebook), vbox);
  widget = gtk_label_new (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), (gint)page), widget);

  scroll = gtk_scrolled_window_new(NULL,NULL);
  gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll) ,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);

  viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport);
  gtk_container_add (GTK_CONTAINER (scroll), viewport);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);

 vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (viewport), vbox);

  return vbox;

}


void filter_changed(GtkWidget * widget, GdkEventButton *ev, Plugin_Filter * filter) {
  GSList * list = filter->icon_group->icons_list;

  while (list) {
	Plugin_Filter * tmp_filter = list->data;
	 safe_gtk_widget_hide(tmp_filter->widget);
         list = list->next;
	}

	/* General Options */
	  if ( filter->widget == NULL) {
  	//Generate when asked to save speed during intitial load

	  filter->widget = build_pane((gchar *)filter->icon_group->data, filter->filtered_allscreen_options_list, filter->filtered_screen_options_list, GINT_TO_POINTER(filter->option_filter), filter->filter_must_not_have, filter->filter_must_have, filter->title);

  	  gtk_box_pack_start (GTK_BOX (InterfaceVbox), filter->widget, FALSE, TRUE, 0);
	  }

      gtk_widget_show_all (filter->widget);


}


Plugin_Filter * get_filter_by_name(IconGroup *group, gchar * name) {
   Plugin_Filter * filter = NULL;

  GSList * list = group->icons_list;
  while (list)
    {
      /* Process */
	Plugin_Filter * tmp_filter = list->data;
  	if (!strcmp(tmp_filter->name, name)) {
   	 filter = tmp_filter;
  	}
      list = list->next;
    }

  return filter;
}

void add_icon_from_filter(Plugin_Filter * filter, IconGroup * new_icon_group) {
  	  add_icon_to_group_table(filter->title, filter->image, new_icon_group, (GFunc *)filter_changed, (gpointer *) filter, FALSE);

}

 void generate_groups(IconGroup * new_icon_group) {


  	GtkWidget * table = gtk_table_new (2, 3, FALSE);

  	gtk_box_pack_start (GTK_BOX (new_icon_group->container), table, FALSE, FALSE, 0);

  	new_icon_group->regenerate_func = (GFunc) *generate_groups;
  	new_icon_group->current_col=0;
  	new_icon_group->current_row=0;
  	new_icon_group->table = table;


	/* Experimental */
	g_slist_foreach(new_icon_group->icons_list, (GFunc) add_icon_from_filter, new_icon_group);

	/* End */
	//g_free(key);
  	gtk_widget_show_all(table);

}


gint remember_page;
void notebook_page_switched(GtkNotebook     *notebook,
                                            GtkNotebookPage *page,
                                            guint            page_num,
                                            gpointer         user_data) {
   //printf("Notebook Page Switched to %d\n", page_num);
   remember_page = page_num;

}


GtkWidget * build_plugin_interface(gchar * plugin_name) {
//printf("build_plugin_interface start\n");
	GtkWidget * hbox;
	GtkWidget * vbox;
  	GtkWidget * hbox2;
	gchar *  plugin_title = NULL;
	GSList * filters_list = NULL;
	gboolean at_least_one_filter = FALSE;

	hbox = gtk_hbox_new (TRUE, 0);

	vbox = gtk_vbox_new (FALSE, 0);
  	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);


	plugin_title = CompizSettings_backend_get_plugin_title(plugin_name);
  	 plugin_sections_group = icon_selection_new_icon_group(g_strdup_printf("Options for: %s", plugin_title));
	 g_free(plugin_title);
  	plugin_sections_group->data = (gpointer *) plugin_name;
 	plugin_sections_group->extra_widgets = NULL;
 	plugin_sections_group->icons_list = NULL;

	//printf("	getting filters list\n");
	filters_list = CompizSettings_backend_get_filter_list(plugin_name);
	//printf("	getting options list\n");

	plugin_sections_group->allscreen_options_list = NULL;
	plugin_sections_group->screen_options_list = NULL;
 	plugin_sections_group->allscreen_options_list = backend_get_list_of_options(plugin_name, "allscreens");
 	plugin_sections_group->screen_options_list = backend_get_list_of_options(plugin_name, "screen0");

	//printf("build_plugin_interface filters loop\n");
	for (; filters_list; filters_list = filters_list->next) {
	  Plugin_Filter * filter = filters_list->data;

	 GSList * filtered_allscreen_options_list = (GSList *) backend_apply_option_filter(plugin_sections_group->allscreen_options_list, (gchar *)plugin_name,  "allscreens", filter->option_filter, filter->filter_must_not_have, filter->filter_must_have);

	 GSList * filtered_screen_options_list = (GSList *) backend_apply_option_filter(plugin_sections_group->screen_options_list, (gchar *)plugin_name,  "screen0", filter->option_filter, filter->filter_must_not_have, filter->filter_must_have);


       if (filtered_allscreen_options_list != NULL || filtered_screen_options_list != NULL) {
	       		filter->icon_group = plugin_sections_group;
	       		filter->filtered_screen_options_list = filtered_screen_options_list;
	       		filter->filtered_allscreen_options_list = filtered_allscreen_options_list;
			filter->widget = NULL;

	  	 plugin_sections_group->icons_list = g_slist_append(plugin_sections_group->icons_list, filter);	
		at_least_one_filter = TRUE;
	}

	}


	//  g_slist_free(screen_options_list);
	//  g_slist_free(allscreen_options_list);


  	generate_groups(plugin_sections_group);
  	gtk_box_pack_start (GTK_BOX (vbox), plugin_sections_group->eventbox, FALSE, FALSE, 0);


	g_slist_free(filters_list);

	hbox2 = gtk_hbox_new (FALSE, 0);
  	gtk_box_pack_start (GTK_BOX (vbox), hbox2, TRUE, TRUE, 0);


  	GtkWidget * eventbox = gtk_event_box_new ();
      	gtk_widget_show (eventbox);
  	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &color);
  	gtk_widget_set_size_request (eventbox, 10, -1);
  	gtk_box_pack_start (GTK_BOX (hbox2), eventbox, FALSE, FALSE, 0);

  	eventbox = gtk_event_box_new ();
      	gtk_widget_show (eventbox);
	GdkColor color;
	gdk_color_parse ("#A99D8C", &color);
  	gtk_widget_modify_bg (GTK_WIDGET(eventbox), GTK_STATE_NORMAL, &color);
  	gtk_widget_set_size_request (eventbox, 1, -1);
  	gtk_box_pack_start (GTK_BOX (hbox2), eventbox, FALSE, FALSE, 0);


	InterfaceVbox = gtk_vbox_new (FALSE, 0);
  	eventbox = gtk_event_box_new ();
      	gtk_widget_show (eventbox);
  	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &color);
  	gtk_widget_set_size_request (eventbox, -1, 1);
  	gtk_box_pack_start (GTK_BOX (InterfaceVbox), eventbox, FALSE, FALSE, 0);

  	gtk_box_pack_start (GTK_BOX (hbox2), InterfaceVbox, TRUE, TRUE, 0);

	/* No options Found */
	if (at_least_one_filter == FALSE) {
	  GtkWidget * label;
	  GtkWidget * image;
  	  GtkWidget * hbox = gtk_hbox_new (FALSE, 0);
  	  GtkWidget * hbox2 = gtk_hbox_new (TRUE, 0);

		image = create_pixmap (window1, "info.png");
  	    gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);

  		label = gtk_label_new ("<b>No configurable options found</b>");
  		gtk_misc_set_padding (GTK_MISC (label), 10, 0);
 		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  	    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  	  gtk_box_pack_start (GTK_BOX (hbox2), hbox, FALSE, FALSE, 0);
  	  gtk_box_pack_start (GTK_BOX (InterfaceVbox), hbox2, TRUE, TRUE, 0);

	  gtk_widget_show_all(hbox2);
	}


	Plugin_Filter * tmp_filter = g_slist_nth_data (plugin_sections_group->icons_list, 0);

	if (tmp_filter != NULL) {
	    filter_changed(NULL, NULL, tmp_filter);
	}

      	gtk_widget_show_all (plugin_sections_group->eventbox);
      	gtk_widget_show (hbox2);
      	gtk_widget_show (InterfaceVbox);
      	gtk_widget_show (vbox);
      	gtk_widget_show (hbox);

   return hbox;
}


void interface_go_back() {
	//printf("Interface go'd back\n");

	if (configuration_vbox != NULL && GTK_IS_WIDGET(configuration_vbox)) {
         gtk_widget_hide (configuration_vbox);
	}

	//printf("Showing icon_selection_vbox\n");
	 gtk_widget_show (icon_selection_vbox);

	//printf("Showing left_vbox->reset_default\n");
	 gtk_widget_show(left_vbox->reset_default);

	//printf("Destorying plugin_sections_group\n");
	if (plugin_sections_group != NULL) {
	  destroy_icon_group(plugin_sections_group);
	  plugin_sections_group = NULL;
	}
	//printf("Removing callbacks\n");

        //safe_gtk_widget_destroy (configuration_vbox);
	backened_remove_callbacks ();

        redraw_groups(icon_groups);

	//printf("Interface gone back\n");

}


void active_plugin_changed(gchar * plugin_name) {
  GtkWidget *interface;
  GtkWidget *scroll_box;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *widget;

       //printf("active_plugin_changed\n");

  	plugin_sections_group = malloc(sizeof(IconGroup));
	plugin_sections_group->extra_widgets = NULL;
	plugin_sections_group->icons_list = NULL;

	configuration_vbox = gtk_vbox_new (TRUE, 0);
        //gtk_widget_set_size_request (configuration_vbox, 690, 500);

  	 scroll_box = gtk_scrolled_window_new (NULL, NULL);
  	 gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_box), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  	 gtk_box_pack_start (GTK_BOX (configuration_vbox), scroll_box, TRUE, TRUE, 0);

 	 vbox = gtk_vbox_new (TRUE, 0);
  	 gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroll_box), vbox);
	  

         //printf("active_plugin_changed build_plugin_interface\n");
	 interface = build_plugin_interface(plugin_name);

          //printf("done active_plugin_changed build_plugin_interface\n");
	
  	 gtk_box_pack_start (GTK_BOX (vbox), interface, TRUE, TRUE, 0);

  	 gtk_box_pack_start (GTK_BOX (main_hbox), configuration_vbox, TRUE, TRUE, 0);

 	g_signal_connect ((gpointer) configuration_vbox, "size_allocate", G_CALLBACK (icons_window_resized), plugin_sections_group);
 
      	gtk_widget_show (scroll_box);
      	gtk_widget_show (vbox);

        gtk_widget_hide (icon_selection_vbox);

      	gtk_widget_show (configuration_vbox);

	/*Add options to left pane */
  	//widget = new_label_button(10, "Enable Wobbly", (GFunc *) interface_change_button_clicked);

	Plugin_Filter_Button * button_data = malloc(sizeof(Plugin_Filter_Button));
	widget = gtk_check_button_new_with_mnemonic("Enable/Disable Plugin");
	if (!data_is_in_gslist(active_plugins, plugin_name)) {
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), FALSE);
	        button_data->checkbox_active = FALSE;
	} else {
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
	        button_data->checkbox_active = TRUE;
	}
	   button_data->plugin_name = plugin_name;
	   button_data->checkbox = widget;

	 hbox = gtk_hbox_new (FALSE, 0);
 	 GtkWidget * alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
  	gtk_widget_set_size_request (alignment, 13, -1);

  	gtk_box_pack_start (GTK_BOX (hbox), alignment, FALSE, FALSE, 0);
  	gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);

	  plugin_sections_group->extra_widgets = g_slist_append(plugin_sections_group->extra_widgets, hbox);
	  g_signal_connect (GTK_WIDGET (widget), "toggled", G_CALLBACK(plugin_toggle_button_pressed), plugin_name);

	  /* Update if changed externally */
	   backend_create_new_callback(COMPIZ_PATH "/general/allscreens/options/active_plugins", (GFunc *) active_plugins_changed_checkbox, (GtkWidget *) button_data);

  	  gtk_box_pack_start (GTK_BOX (left_vbox->common_tasks), hbox, FALSE, FALSE, 0);
      	  gtk_widget_show_all (left_vbox->common_tasks);


	/* Hide some widgets in common tasks */
      	  gtk_widget_hide (left_vbox->reset_default);

}





